@@ -1,6 +1,7 @@
 """Configuration classes."""
 from __future__ import annotations
 
+import dataclasses
 import enum
 import os
 import sys
@@ -10,6 +11,7 @@ from .util import (
     display,
     verify_sys_executable,
     version_to_str,
+    type_guard,
 )
 
 from .util_common import (
@@ -47,43 +49,40 @@ class TerminateMode(enum.Enum):
         return self.name.lower()
 
 
-class ParsedRemote:
-    """A parsed version of a "remote" string."""
-    def __init__(self, arch, platform, version):  # type: (t.Optional[str], str, str) -> None
-        self.arch = arch
-        self.platform = platform
-        self.version = version
+@dataclasses.dataclass(frozen=True)
+class ModulesConfig:
+    """Configuration for modules."""
+    python_requires: str
+    python_versions: tuple[str, ...]
+    controller_only: bool
 
-    @staticmethod
-    def parse(value):  # type: (str) -> t.Optional['ParsedRemote']
-        """Return a ParsedRemote from the given value or None if the syntax is invalid."""
-        parts = value.split('/')
 
-        if len(parts) == 2:
-            arch = None
-            platform, version = parts
-        elif len(parts) == 3:
-            arch, platform, version = parts
-        else:
-            return None
-
-        return ParsedRemote(arch, platform, version)
+@dataclasses.dataclass(frozen=True)
+class ContentConfig:
+    """Configuration for all content."""
+    modules: ModulesConfig
+    python_versions: tuple[str, ...]
+    py2_support: bool
 
 
 class EnvironmentConfig(CommonConfig):
     """Configuration common to all commands which execute in an environment."""
-    def __init__(self, args, command):  # type: (t.Any, str) -> None
+    def __init__(self, args: t.Any, command: str) -> None:
         super().__init__(args, command)
 
-        self.host_settings = args.host_settings  # type: HostSettings
-        self.host_path = args.host_path  # type: t.Optional[str]
-        self.containers = args.containers  # type: t.Optional[str]
-        self.pypi_proxy = args.pypi_proxy  # type: bool
-        self.pypi_endpoint = args.pypi_endpoint  # type: t.Optional[str]
+        self.host_settings: HostSettings = args.host_settings
+        self.host_path: t.Optional[str] = args.host_path
+        self.containers: t.Optional[str] = args.containers
+        self.pypi_proxy: bool = args.pypi_proxy
+        self.pypi_endpoint: t.Optional[str] = args.pypi_endpoint
+
+        # Populated by content_config.get_content_config on the origin.
+        # Serialized and passed to delegated instances to avoid parsing a second time.
+        self.content_config: t.Optional[ContentConfig] = None
 
         # Set by check_controller_python once HostState has been created by prepare_profiles.
         # This is here for convenience, to avoid needing to pass HostState to some functions which already have access to EnvironmentConfig.
-        self.controller_python = None  # type: t.Optional[PythonConfig]
+        self.controller_python: t.Optional[PythonConfig] = None
         """
         The Python interpreter used by the controller.
         Only available after delegation has been performed or skipped (if delegation is not required).
@@ -96,32 +95,34 @@ class EnvironmentConfig(CommonConfig):
                 not isinstance(self.controller, OriginConfig)
                 or isinstance(self.controller.python, VirtualPythonConfig)
                 or self.controller.python.version != version_to_str(sys.version_info[:2])
-                or verify_sys_executable(self.controller.python.path)
+                or bool(verify_sys_executable(self.controller.python.path))
             )
 
-        self.docker_network = args.docker_network  # type: t.Optional[str]
-        self.docker_terminate = args.docker_terminate  # type: t.Optional[TerminateMode]
+        self.docker_network: t.Optional[str] = args.docker_network
+        self.docker_terminate: t.Optional[TerminateMode] = args.docker_terminate
 
-        self.remote_endpoint = args.remote_endpoint  # type: t.Optional[str]
-        self.remote_stage = args.remote_stage  # type: t.Optional[str]
-        self.remote_terminate = args.remote_terminate  # type: t.Optional[TerminateMode]
+        self.remote_endpoint: t.Optional[str] = args.remote_endpoint
+        self.remote_stage: t.Optional[str] = args.remote_stage
+        self.remote_terminate: t.Optional[TerminateMode] = args.remote_terminate
 
-        self.prime_containers = args.prime_containers  # type: bool
+        self.prime_containers: bool = args.prime_containers
 
-        self.requirements = args.requirements  # type: bool
+        self.requirements: bool = args.requirements
 
-        self.delegate_args = []  # type: t.List[str]
+        self.delegate_args: list[str] = []
 
-        def host_callback(files):  # type: (t.List[t.Tuple[str, str]]) -> None
+        def host_callback(files: list[tuple[str, str]]) -> None:
             """Add the host files to the payload file list."""
             config = self
 
             if config.host_path:
                 settings_path = os.path.join(config.host_path, 'settings.dat')
                 state_path = os.path.join(config.host_path, 'state.dat')
+                config_path = os.path.join(config.host_path, 'config.dat')
 
                 files.append((os.path.abspath(settings_path), settings_path))
                 files.append((os.path.abspath(state_path), state_path))
+                files.append((os.path.abspath(config_path), config_path))
 
         data_context().register_payload_callback(host_callback)
 
@@ -132,16 +133,16 @@ class EnvironmentConfig(CommonConfig):
             display.warning('The --no-pip-check option is deprecated and has no effect. It will be removed in a future version of ansible-test.')
 
     @property
-    def controller(self):  # type: () -> ControllerHostConfig
+    def controller(self) -> ControllerHostConfig:
         """Host configuration for the controller."""
         return self.host_settings.controller
 
     @property
-    def targets(self):  # type: () -> t.List[HostConfig]
+    def targets(self) -> list[HostConfig]:
         """Host configuration for the targets."""
         return self.host_settings.targets
 
-    def only_target(self, target_type):  # type: (t.Type[THostConfig]) -> THostConfig
+    def only_target(self, target_type: t.Type[THostConfig]) -> THostConfig:
         """
         Return the host configuration for the target.
         Requires that there is exactly one target of the specified type.
@@ -158,22 +159,20 @@ class EnvironmentConfig(CommonConfig):
 
         return target
 
-    def only_targets(self, target_type):  # type: (t.Type[THostConfig]) -> t.List[THostConfig]
+    def only_targets(self, target_type: t.Type[THostConfig]) -> list[THostConfig]:
         """
         Return a list of target host configurations.
-        Requires that there are one or more targets, all of the specified type.
+        Requires that there are one or more targets, all the specified type.
         """
         if not self.targets:
             raise Exception('There must be one or more targets.')
 
-        for target in self.targets:
-            if not isinstance(target, target_type):
-                raise Exception(f'Target is {type(target_type)} instead of {target_type}.')
+        assert type_guard(self.targets, target_type)
 
-        return self.targets
+        return t.cast(list[THostConfig], self.targets)
 
     @property
-    def target_type(self):  # type: () -> t.Type[HostConfig]
+    def target_type(self) -> t.Type[HostConfig]:
         """
         The true type of the target(s).
         If the target is the controller, the controller type is returned.
@@ -194,36 +193,36 @@ class EnvironmentConfig(CommonConfig):
 
 class TestConfig(EnvironmentConfig):
     """Configuration common to all test commands."""
-    def __init__(self, args, command):  # type: (t.Any, str) -> None
+    def __init__(self, args: t.Any, command: str) -> None:
         super().__init__(args, command)
 
-        self.coverage = args.coverage  # type: bool
-        self.coverage_check = args.coverage_check  # type: bool
-        self.include = args.include or []  # type: t.List[str]
-        self.exclude = args.exclude or []  # type: t.List[str]
-        self.require = args.require or []  # type: t.List[str]
+        self.coverage: bool = args.coverage
+        self.coverage_check: bool = args.coverage_check
+        self.include: list[str] = args.include or []
+        self.exclude: list[str] = args.exclude or []
+        self.require: list[str] = args.require or []
 
-        self.changed = args.changed  # type: bool
-        self.tracked = args.tracked  # type: bool
-        self.untracked = args.untracked  # type: bool
-        self.committed = args.committed  # type: bool
-        self.staged = args.staged  # type: bool
-        self.unstaged = args.unstaged  # type: bool
-        self.changed_from = args.changed_from  # type: str
-        self.changed_path = args.changed_path  # type: t.List[str]
-        self.base_branch = args.base_branch  # type: str
+        self.changed: bool = args.changed
+        self.tracked: bool = args.tracked
+        self.untracked: bool = args.untracked
+        self.committed: bool = args.committed
+        self.staged: bool = args.staged
+        self.unstaged: bool = args.unstaged
+        self.changed_from: str = args.changed_from
+        self.changed_path: list[str] = args.changed_path
+        self.base_branch: str = args.base_branch
 
-        self.lint = getattr(args, 'lint', False)  # type: bool
-        self.junit = getattr(args, 'junit', False)  # type: bool
-        self.failure_ok = getattr(args, 'failure_ok', False)  # type: bool
+        self.lint: bool = getattr(args, 'lint', False)
+        self.junit: bool = getattr(args, 'junit', False)
+        self.failure_ok: bool = getattr(args, 'failure_ok', False)
 
         self.metadata = Metadata.from_file(args.metadata) if args.metadata else Metadata()
-        self.metadata_path = None
+        self.metadata_path: t.Optional[str] = None
 
         if self.coverage_check:
             self.coverage = True
 
-        def metadata_callback(files):  # type: (t.List[t.Tuple[str, str]]) -> None
+        def metadata_callback(files: list[tuple[str, str]]) -> None:
             """Add the metadata file to the payload file list."""
             config = self
 
@@ -235,29 +234,34 @@ class TestConfig(EnvironmentConfig):
 
 class ShellConfig(EnvironmentConfig):
     """Configuration for the shell command."""
-    def __init__(self, args):  # type: (t.Any) -> None
+    def __init__(self, args: t.Any) -> None:
         super().__init__(args, 'shell')
 
-        self.raw = args.raw  # type: bool
+        self.cmd: list[str] = args.cmd
+        self.raw: bool = args.raw
+        self.check_layout = self.delegate  # allow shell to be used without a valid layout as long as no delegation is required
+        self.interactive = sys.stdin.isatty() and not args.cmd  # delegation should only be interactive when stdin is a TTY and no command was given
+        self.export: t.Optional[str] = args.export
+        self.display_stderr = True
 
 
 class SanityConfig(TestConfig):
     """Configuration for the sanity command."""
-    def __init__(self, args):  # type: (t.Any) -> None
+    def __init__(self, args: t.Any) -> None:
         super().__init__(args, 'sanity')
 
-        self.test = args.test  # type: t.List[str]
-        self.skip_test = args.skip_test  # type: t.List[str]
-        self.list_tests = args.list_tests  # type: bool
-        self.allow_disabled = args.allow_disabled  # type: bool
-        self.enable_optional_errors = args.enable_optional_errors  # type: bool
-        self.keep_git = args.keep_git  # type: bool
-        self.prime_venvs = args.prime_venvs  # type: bool
+        self.test: list[str] = args.test
+        self.skip_test: list[str] = args.skip_test
+        self.list_tests: bool = args.list_tests
+        self.allow_disabled: bool = args.allow_disabled
+        self.enable_optional_errors: bool = args.enable_optional_errors
+        self.keep_git: bool = args.keep_git
+        self.prime_venvs: bool = args.prime_venvs
 
-        self.info_stderr = self.lint
+        self.display_stderr = self.lint or self.list_tests
 
         if self.keep_git:
-            def git_callback(files):  # type: (t.List[t.Tuple[str, str]]) -> None
+            def git_callback(files: list[tuple[str, str]]) -> None:
                 """Add files from the content root .git directory to the payload file list."""
                 for dirpath, _dirnames, filenames in os.walk(os.path.join(data_context().content.root, '.git')):
                     paths = [os.path.join(dirpath, filename) for filename in filenames]
@@ -268,34 +272,34 @@ class SanityConfig(TestConfig):
 
 class IntegrationConfig(TestConfig):
     """Configuration for the integration command."""
-    def __init__(self, args, command):  # type: (t.Any, str) -> None
+    def __init__(self, args: t.Any, command: str) -> None:
         super().__init__(args, command)
 
-        self.start_at = args.start_at  # type: str
-        self.start_at_task = args.start_at_task  # type: str
-        self.allow_destructive = args.allow_destructive  # type: bool
-        self.allow_root = args.allow_root  # type: bool
-        self.allow_disabled = args.allow_disabled  # type: bool
-        self.allow_unstable = args.allow_unstable  # type: bool
-        self.allow_unstable_changed = args.allow_unstable_changed  # type: bool
-        self.allow_unsupported = args.allow_unsupported  # type: bool
-        self.retry_on_error = args.retry_on_error  # type: bool
-        self.continue_on_error = args.continue_on_error  # type: bool
-        self.debug_strategy = args.debug_strategy  # type: bool
-        self.changed_all_target = args.changed_all_target  # type: str
-        self.changed_all_mode = args.changed_all_mode  # type: str
-        self.list_targets = args.list_targets  # type: bool
+        self.start_at: str = args.start_at
+        self.start_at_task: str = args.start_at_task
+        self.allow_destructive: bool = args.allow_destructive
+        self.allow_root: bool = args.allow_root
+        self.allow_disabled: bool = args.allow_disabled
+        self.allow_unstable: bool = args.allow_unstable
+        self.allow_unstable_changed: bool = args.allow_unstable_changed
+        self.allow_unsupported: bool = args.allow_unsupported
+        self.retry_on_error: bool = args.retry_on_error
+        self.continue_on_error: bool = args.continue_on_error
+        self.debug_strategy: bool = args.debug_strategy
+        self.changed_all_target: str = args.changed_all_target
+        self.changed_all_mode: str = args.changed_all_mode
+        self.list_targets: bool = args.list_targets
         self.tags = args.tags
         self.skip_tags = args.skip_tags
         self.diff = args.diff
-        self.no_temp_workdir = args.no_temp_workdir
-        self.no_temp_unicode = args.no_temp_unicode
+        self.no_temp_workdir: bool = args.no_temp_workdir
+        self.no_temp_unicode: bool = args.no_temp_unicode
 
         if self.list_targets:
             self.explain = True
-            self.info_stderr = True
+            self.display_stderr = True
 
-    def get_ansible_config(self):  # type: () -> str
+    def get_ansible_config(self) -> str:
         """Return the path to the Ansible config for the given config."""
         ansible_config_relative_path = os.path.join(data_context().content.integration_path, '%s.cfg' % self.command)
         ansible_config_path = os.path.join(data_context().content.root, ansible_config_relative_path)
@@ -312,33 +316,33 @@ TIntegrationConfig = t.TypeVar('TIntegrationConfig', bound=IntegrationConfig)
 
 class PosixIntegrationConfig(IntegrationConfig):
     """Configuration for the posix integration command."""
-    def __init__(self, args):  # type: (t.Any) -> None
+    def __init__(self, args: t.Any) -> None:
         super().__init__(args, 'integration')
 
 
 class WindowsIntegrationConfig(IntegrationConfig):
     """Configuration for the windows integration command."""
-    def __init__(self, args):  # type: (t.Any) -> None
+    def __init__(self, args: t.Any) -> None:
         super().__init__(args, 'windows-integration')
 
 
 class NetworkIntegrationConfig(IntegrationConfig):
     """Configuration for the network integration command."""
-    def __init__(self, args):  # type: (t.Any) -> None
+    def __init__(self, args: t.Any) -> None:
         super().__init__(args, 'network-integration')
 
-        self.testcase = args.testcase  # type: str
+        self.testcase: str = args.testcase
 
 
 class UnitsConfig(TestConfig):
     """Configuration for the units command."""
-    def __init__(self, args):  # type: (t.Any) -> None
+    def __init__(self, args: t.Any) -> None:
         super().__init__(args, 'units')
 
-        self.collect_only = args.collect_only  # type: bool
-        self.num_workers = args.num_workers  # type: int
+        self.collect_only: bool = args.collect_only
+        self.num_workers: int = args.num_workers
 
-        self.requirements_mode = getattr(args, 'requirements_mode', '')  # type: str
+        self.requirements_mode: str = getattr(args, 'requirements_mode', '')
 
         if self.requirements_mode == 'only':
             self.requirements = True
