@@ -1,5 +1,4 @@
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 import contextlib
 import fnmatch
@@ -29,6 +28,7 @@ def assemble_files_to_ship(complete_file_list):
         'hacking/tests/*',
         'hacking/ticket_stubs/*',
         'test/sanity/code-smell/botmeta.*',
+        'test/sanity/code-smell/release-names.*',
         'test/utils/*',
         'test/utils/*/*',
         'test/utils/*/*/*',
@@ -47,19 +47,22 @@ def assemble_files_to_ship(complete_file_list):
         'hacking/cgroup_perf_recap_graph.py',
         'hacking/create_deprecated_issues.py',
         'hacking/deprecated_issue_template.md',
+        'hacking/create_deprecation_bug_reports.py',
         'hacking/fix_test_syntax.py',
         'hacking/get_library.py',
         'hacking/metadata-tool.py',
         'hacking/report.py',
         'hacking/return_skeleton_generator.py',
         'hacking/test-module',
-        'hacking/test-module.py',
         'test/support/README.md',
+        'test/lib/ansible_test/_internal/commands/sanity/bin_symlinks.py',
+        'test/lib/ansible_test/_internal/commands/sanity/integration_aliases.py',
         '.cherry_picker.toml',
         '.mailmap',
         # Generated as part of a build step
         'docs/docsite/rst/conf.py',
         'docs/docsite/rst/index.rst',
+        'docs/docsite/rst/dev_guide/index.rst',
         # Possibly should be included
         'examples/scripts/uptime.py',
         'examples/scripts/my_test.py',
@@ -74,22 +77,27 @@ def assemble_files_to_ship(complete_file_list):
         'hacking/env-setup',
         'hacking/env-setup.fish',
         'MANIFEST',
+        'setup.cfg',
+        # docs for test files not included in sdist
+        'docs/docsite/rst/dev_guide/testing/sanity/bin-symlinks.rst',
+        'docs/docsite/rst/dev_guide/testing/sanity/botmeta.rst',
+        'docs/docsite/rst/dev_guide/testing/sanity/integration-aliases.rst',
+        'docs/docsite/rst/dev_guide/testing/sanity/release-names.rst',
     ))
 
     # These files are generated and then intentionally added to the sdist
 
     # Manpages
+    ignore_script = ('ansible-connection', 'ansible-test')
     manpages = ['docs/man/man1/ansible.1']
     for dirname, dummy, files in os.walk('bin'):
         for filename in files:
-            path = os.path.join(dirname, filename)
-            if os.path.islink(path):
-                if os.readlink(path) == 'ansible':
-                    manpages.append('docs/man/man1/%s.1' % filename)
+            if filename in ignore_script:
+                continue
+            manpages.append('docs/man/man1/%s.1' % filename)
 
     # Misc
     misc_generated_files = [
-        'SYMLINK_CACHE.json',
         'PKG-INFO',
     ]
 
@@ -110,7 +118,11 @@ def assemble_files_to_install(complete_file_list):
     """
     This looks for all of the files which should show up in an installation of ansible
     """
-    ignore_patterns = tuple()
+    ignore_patterns = (
+        # Tests excluded from sdist
+        'test/lib/ansible_test/_internal/commands/sanity/bin_symlinks.py',
+        'test/lib/ansible_test/_internal/commands/sanity/integration_aliases.py',
+    )
 
     pkg_data_files = []
     for path in complete_file_list:
@@ -161,14 +173,15 @@ def clean_repository(file_list):
 
 def create_sdist(tmp_dir):
     """Create an sdist in the repository"""
-    create = subprocess.Popen(
+    create = subprocess.run(
         ['make', 'snapshot', 'SDIST_DIR=%s' % tmp_dir],
-        stdout=subprocess.PIPE,
-        stderr=subprocess.PIPE,
-        universal_newlines=True,
+        stdin=subprocess.DEVNULL,
+        capture_output=True,
+        text=True,
+        check=False,
     )
 
-    stderr = create.communicate()[1]
+    stderr = create.stderr
 
     if create.returncode != 0:
         raise Exception('make snapshot failed:\n%s' % stderr)
@@ -209,22 +222,24 @@ def extract_sdist(sdist_path, tmp_dir):
 
 def install_sdist(tmp_dir, sdist_dir):
     """Install the extracted sdist into the temporary directory"""
-    install = subprocess.Popen(
+    install = subprocess.run(
         ['python', 'setup.py', 'install', '--root=%s' % tmp_dir],
-        stdout=subprocess.PIPE,
-        stderr=subprocess.PIPE,
-        universal_newlines=True,
+        stdin=subprocess.DEVNULL,
+        capture_output=True,
+        text=True,
         cwd=os.path.join(tmp_dir, sdist_dir),
+        check=False,
     )
 
-    stdout, stderr = install.communicate()
+    stdout, stderr = install.stdout, install.stderr
 
     if install.returncode != 0:
         raise Exception('sdist install failed:\n%s' % stderr)
 
     # Determine the prefix for the installed files
-    match = re.search('^creating (%s/.*?/(?:site|dist)-packages)/ansible$' %
+    match = re.search('^copying .* -> (%s/.*?/(?:site|dist)-packages)/ansible$' %
                       tmp_dir, stdout, flags=re.M)
+
     return match.group(1)
 
 
@@ -256,12 +271,19 @@ def check_sdist_files_are_wanted(sdist_dir, to_ship_files):
             dirname = ''
 
         for filename in files:
+            if filename == 'setup.cfg':
+                continue
+
             path = os.path.join(dirname, filename)
             if path not in to_ship_files:
+
                 if fnmatch.fnmatch(path, 'changelogs/CHANGELOG-v2.[0-9]*.rst'):
                     # changelog files are expected
                     continue
 
+                if fnmatch.fnmatch(path, 'lib/ansible_core.egg-info/*'):
+                    continue
+
                 # FIXME: ansible-test doesn't pass the paths of symlinks to us so we aren't
                 # checking those
                 if not os.path.islink(os.path.join(sdist_dir, path)):
@@ -282,7 +304,7 @@ def check_installed_contains_expected(install_dir, to_install_files):
 
 
 EGG_RE = re.compile('ansible[^/]+\\.egg-info/(PKG-INFO|SOURCES.txt|'
-                    'dependency_links.txt|not-zip-safe|requires.txt|top_level.txt)$')
+                    'dependency_links.txt|not-zip-safe|requires.txt|top_level.txt|entry_points.txt)$')
 
 
 def check_installed_files_are_wanted(install_dir, to_install_files):
